/***************************************************************************
                          hp4470c.h(c)  -  description
                             -------------------
    begin                : Mon Jan 13 2003
    copyright            : (C) 2003 by johanneshub
    email                : johanneshub@foni.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*
    hp4470.h version 0.14a  26.01.2003 Johannes Hub

    This source is based of the file register.h from Michael Cohen

    History:
    0.14b3 06.08.03 Johannes Hub
    .....
    
    0.14a 26.01.03 Johannes Hub
    Regeditor added for movement investigation.
    scan_75dpi_gray added, but
    it doesn't work at the moment.I see this problem by all
    resolutions at the moment

    0.14 14.01.03 Johannes Hub
    Moved into KDevelop 2.1.3, scan_150dpi_16_color added, but
    it doesn't work at the moment.I see this problem by all
    resolutions at the moment

    0.13 05.01.03 Johannes Hub
    I'm unsing now a speciale patched version from snoopy ver 1.3.

    0.12 16.12.02 Johannes Hub
    Full Scanner callibriation included.

    0.11 06.11.02 Johannes Hub
    Start of my investigation, the version from Michael doesn't work,
    the lamp is off by scanning.
    I'm using the out put from snoopy ver 1.3 to modify the source.

*/
#include <time.h>
#include "rts8891.h"

/****************************************************************************/
/* utility function to show a hexdump of a buffer */
void DumpHex(unsigned char *pabData, int iLen, int iWidth){
/****************************************************************************/
  int i;
  printf("----");
  for (i = 0; i < iWidth; i++)
    printf("--%x",i);

  for (i = 0; i < iLen; i++) {
    if ((i % iWidth) == 0) {
      printf("\n%04X", i);
    }
    printf(" %02X", pabData[i]);
  }
  printf("\n");
}




/* Here are functions to interface to the scanner */

/****************************************************************************/
char read_bulk_callibriation_data(char *data, int count)
/****************************************************************************/
{
  char init[]={0x81,0x00,0x08,0x18};
  init[2]=(unsigned char)(count>>8); /*split count over these bytes.*/
  init[3]=(unsigned char)count;

  write_usb(device,0x02,init,4,TIMEOUT);
  if (count > 2048){
    read_usb(device,0x81,data,2048,TIMEOUT);
    read_usb(device,0x81,data+2048,count-2048,TIMEOUT);
  } else read_usb(device,0x81,data,count,TIMEOUT);

  return(0);
};


/****************************************************************************/
char read_all_registers(char *data)
/****************************************************************************/
{
  char init[]={0x80,0x00,0x00,0xf4};

  write_usb(device,0x02,init,4,TIMEOUT);
  read_usb(device,0x81,data,192,TIMEOUT);
  read_usb(device,0x81,data+192,52,TIMEOUT);

  return(0);
};


/****************************************************************************/
char read_bulk_registers(char *data,int count,unsigned char reg)
/****************************************************************************/
{ /* Reads count registers into data - return error code. */
  char init[]={0x80,reg,0x00,0x01};

  init[2]=(unsigned char)(count>>8); //split count over these bytes.
  init[3]=(unsigned char)count;

  write_usb(device,0x02,init,4,TIMEOUT);
  read_usb(device,0x81,data,count,TIMEOUT);
  return(0);
};


/****************************************************************************/
char write_bulk_registers(char *data,int count,unsigned char reg)
/****************************************************************************/
{ /* writes count registers into data - return error code. */
  char init[]={0x88,reg,0x00,0x01};

  char *request=malloc(count + 4);     /* Compose a request string. */

  init[2]=(unsigned char)(count>>8); /* split count over these bytes.*/
  init[3]=(unsigned char)count;

  memcpy(request,init,4);        /* stick them together */
  memcpy(request+4,data,count);
  write_usb(device,0x02,request,count+4,TIMEOUT);
  free(request);                  /* clean up */
  return(0);
};


/****************************************************************************/
unsigned char read_reg(unsigned char reg)
/****************************************************************************/
{ /* Reads a single register and returns it */
  unsigned char data[]={0x80,reg,0x00,0x01};

  write_usb(device,0x02,data,4,TIMEOUT);
  read_usb(device,0x81,data,1,TIMEOUT);
  return(data[0]);
};


/****************************************************************************/
unsigned int read_double_reg(unsigned char reg)
/****************************************************************************/
{
  unsigned int n;
  unsigned char data[]={0x80,reg,0x00,0x02};

  write_usb(device,0x02,data,4,TIMEOUT);
  read_usb(device,0x81,data,2,TIMEOUT);

  n = data[1];
  n = n + ((data[0] <<8) & 0xFF00);
  return(n);
};

/****************************************************************************/
int write_reg(unsigned char reg,unsigned char value)
/****************************************************************************/
{ /* writes a single register */
  int wrote;

  char data[]={0x88,reg,0x00,0x01,value};
  wrote = write_usb(device,0x02,data,5,TIMEOUT);
  return( !(wrote == 5) );
};


/****************************************************************************/
void set_double_reg( unsigned char reg, unsigned char c1, unsigned char c2 )
/****************************************************************************/
{
  unsigned char regs[5];

  regs[0] = c1;
  regs[1] = c2;
  write_bulk_registers( regs, 0x02, reg);
};


/****************************************************************************/
int test_reg( unsigned char reg, unsigned char await, int number ){
/****************************************************************************/
  /* Reads a single register and check it */
  unsigned char v;
  v = read_reg(reg);
  if ( v != await ) printf ("%d Test single reg %x : await %x, read %x\n",number,reg,await,v);
  return (v != await);
};

/****************************************************************************/
int test_double_reg( unsigned char reg, unsigned char await1, unsigned char await2, int number ){
/****************************************************************************/
  /*/Reads a double register and check it */
  unsigned char data[]={0x80,reg,0x00,0x02};

  write_usb(device,0x02,data,4,TIMEOUT);
  read_usb(device,0x81,data,2,TIMEOUT);

  if (( data[0] != await1 ) || ( data[1] != await2 ))
  {
    printf ("%d Test double reg %x : await %x %x, read %x %x\n",number,reg,await1,await2,data[0],data[1]);
    return (0);
  }
  else return (1);
};


/****************************************************************************/
int test_three_regs(unsigned char reg,unsigned char await1,unsigned char await2,
    unsigned char await3,int number){
/****************************************************************************/
  /*/Reads a register and check it */
  unsigned char data[]={0x80,reg,0x00,0x03};

  write_usb(device,0x02,data,4,TIMEOUT);
  read_usb(device,0x81,data,3,TIMEOUT);

  if (( data[0] != await1 ) || ( data[1] != await2 )  || ( data[2] != await3 ))
  {
    printf ("%d Test double reg %x : await %x %x %x, read %x %x %x\n",number,reg,await1,await2,await3,
             data[0],data[1],data[2]);
    return (0);
  }
  else return (1);
};


/****************************************************************************/
void getLCDvalues(void)
/****************************************************************************/
{
  unsigned char data[]={0x80,LCD_CONTROL1,0x00,0x02};

  write_usb(device,0x02,data,4,TIMEOUT);
  read_usb(device,0x81,data,2,TIMEOUT);
  LCDvalues[0] = data[0];
  LCDvalues[1] = data[1];
};


/****************************************************************************/
void getLEDstatus(void)
/****************************************************************************/
{
  unsigned char data[]={0x80,LEDSTATUS,0x00,0x02};

  write_usb(device,0x02,data,4,TIMEOUT);
  read_usb(device,0x81,data,2,TIMEOUT);
  LEDstatus[0] = data[0];
  LEDstatus[1] = data[1];
};


/****************************************************************************/
void setLEDstatus( unsigned char c1, unsigned char c2)
/****************************************************************************/
{
  unsigned char regs[5];

  regs[0] = c1;
  regs[1] = c2;
  write_bulk_registers( regs, 0x02, LEDSTATUS);
};


/****************************************************************************/
unsigned char RT_GetMainBoardID_call(void)
/****************************************************************************/
{
  return(read_reg(MAINBOARD_ID) & 0x0f );
}


/****************************************************************************/
unsigned char GetMicroswitch(void)
/****************************************************************************/
{
  return(read_reg(MICROSWITCH) & 0x02 );
}


/****************************************************************************/
void send_moving_seq(void) {
/****************************************************************************/
    write_reg(MOVE_START_STOP,2);
    write_reg(MOVE_START_STOP,2);
    write_reg(MOVE_START_STOP,0);
    write_reg(MOVE_START_STOP,0);
    write_reg(MOVE_START_STOP,8);
    write_reg(MOVE_START_STOP,8);
/*
    while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?

    printf("done\n");
    while((read_reg(MOVE_START_STOP) & 0x08) && !(read_reg(MICROSWITCH) & 0x02)) usleep(1000000); //are we there yet?
*/
};


/****************************************************************************/
void test_regs( unsigned char *write_regs , char *filename){
/****************************************************************************/
/* only for development */
/* i use this procedure only to write the different regs into a file */
  int n;
  unsigned char read_registers[0xff];
  FILE *FD;

  FD=fopen(filename,"w");
  fprintf(FD,"unsigned char %s[] = {",filename);

  read_all_registers(read_registers);

  printf("compare regs:          reg read write\n");
  for (n=0; n<0xff; n++){
    if (!( read_registers[n] == *write_regs )){
      printf("arrays different by reg, %x, %x, %x \n",n,read_registers[n],*write_regs);
      fprintf(FD,"0x%x,0x%x,",n,*write_regs);
    }
  write_regs ++;
  };
  fprintf(FD,"0x%x,0x%x,",0,0);
  fprintf(FD,"};\n");
  fclose(FD);
};


/****************************************************************************/
unsigned int check_scanner(void)
/****************************************************************************/
{ /* check of the hp4470c is plugged in */
  unsigned char reg;

  reg = read_reg(0x00);
  reg = (reg & 0xe5 );
   if (reg  == 0xe5 )
    return(1);
  else return(0);
}


/****************************************************************************/
void set_display( unsigned char c )  // 0x00 .. 0x01
/****************************************************************************/
{  /* it works 00..01, JoHu 20.11.02 */
   unsigned char regs[5];
                             /* 00         01        , no more implemented */
   unsigned char LcdValues []={ 0xBE,0x7E, 0x3A,0x72 };

   if (c < 2){
      regs[0] = LcdValues [c*2];
      regs[1] = LcdValues [(c*2)+1] | 0x80;
      write_reg(MOVE_START_STOP,0);
      write_reg(0xda,0xa7);
      write_bulk_registers( regs, 0x02, LCD_CONTROL1);
   };
};

/****************************************************************************/
unsigned int check_button()
/****************************************************************************/
{ /* Returns which buttons are pressed.
     HP seems to be doing some strange translations here, but its probably
     because they are trying to make the results compatible with other
     scanners. We dont worry about that too much */
  return(read_reg(BUTTON_1) | (read_reg(BUTTON_2)<<8));
};

/****************************************************************************/
int step_size(int resolution)
/****************************************************************************/
{ /* Return step size. HP only allow a small selection of resolutions. Is it possible to adjust this stepsize to get more resolutions? */
    int step;
    switch(resolution) {
    case 1200:  step=0;
                break;
    case 600:   step=5;
                break;
    case 400:   step=10;
                break;
    case 300:   step=16;
                break;
    case 200:   step=0x14;
                break;
    case 150:   step=0x19;
                break;
    case 100:   step=0x1e;
                break;
    case 75:    step=0x23;
                break;
    default:    step=resolution;
    };
  return(step);
};

/****************************************************************************/
void init_array(int array, int write_it)
/****************************************************************************/
{
     switch (array){
     case 0x01: memcpy(registers,myinit_1,0xf4); // in use
             break;
     case 0x05: memcpy(registers,myinit_5,0xf4); // in use
             break;
     case 0x14: memcpy(registers,myinit20,0xf4); // in use  (20)
             break;
     case 0x1f: memcpy(registers,my__move_back,0xf4); // in use (31)
             break;
#ifdef REG_EDIT
/* from here test */
     case 0x20: memcpy(registers,pv75dpi_g20,0xf4);
             break;
     case 0x21: memcpy(registers,pv75dpi_g21,0xf4);
             break;
     case 0x22: memcpy(registers,pv75dpi_g22,0xf4);
             break;
     case 0x23: memcpy(registers,myinit33,0xf4);  //pv75dpi_g23
             break;
     case 0x24: memcpy(registers,pv75dpi_g24,0xf4);
             break;
     case 0x25: memcpy(registers,pv75dpi_g25,0xf4);
             break;

     case 0x30: memcpy(registers,myscan_1,0xf4);
             break;
     case 0x31: memcpy(registers,myscan_2,0xf4);
             break;
     case 0x99: memcpy(registers,pv0dpi_g25,0xf4);
             break;

#endif
#ifdef DEBUG
     default: printf("init_array : %d not found!\n",array);
#endif
     }
     if (write_it){
     write_bulk_registers(registers,0xb3,0);
     write_bulk_registers(registers+0xb3,0x3f,0xb4); }
};


/****************************************************************************/
void send_my_values(unsigned char *my_values,unsigned char *verify){
/****************************************************************************/
/* read registers, patch them and write out them */
  unsigned char v1,v2;

  read_all_registers(registers);

  v1 = *my_values++;
  v2 = *my_values++;

  registers[v1] = v2;
  while(*my_values){
    v1 = *my_values++;
    v2 = *my_values++;
    registers[v1] = v2;
/*    printf("0x%x,0x%x\n",v1,v2); */
  }
/*
  for (n=0; n<0xff; n++){
    if (!( registers[n] == *verify )){
      printf("error, arrays different by reg, %x, %x, %x \n",n,registers[n],*verify);
    }
    verify ++;
  };
*/
  write_bulk_registers(registers,0xb3,0);
  write_bulk_registers(registers+0xb3,0x3f,0xb4);
};


/****************************************************************************/
void parch_my_values(unsigned char *my_values,unsigned char *verify){
/****************************************************************************/
/* read registers and patch them */
  unsigned char v1,v2;

  read_all_registers(registers);
/*  printf("patching registers\n"); */
  v1 = *my_values++;
  v2 = *my_values++;

  registers[v1] = v2;
  while(*my_values){
    v1 = *my_values++;
    v2 = *my_values++;
    registers[v1] = v2;
/*    printf("0x%x,0x%x\n",v1,v2); */
  }
}


/****************************************************************************/
void switch_off(void)
/****************************************************************************/
{ /* this is the sequenz to switch off the scanner with the on/off button */
  test_reg( 0x1a, 0x00,10 );
  test_reg( 0x1d, 0x22,11 );
  test_reg( 0x1d, 0x22,12 );
  set_double_reg(0x10,0x08,0x28);  /* switch off the little green LEDs for color selector */
  test_reg( 0xb2, 0x02,13 );
  write_reg(0xb2,0x00);   /* switch off the green power LED */
  write_reg(0xb1,0x00);     /* ?? */
  write_reg(0xb3,0x04); /* switch off lamp, LCD and power LED */
  write_reg(0xb3,0x04);
};


/****************************************************************************/
void switch_on(void)
/****************************************************************************/
{ /* this is the sequenz to switch on the scanner with the on/off button */
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  test_reg( 0xb2,0x00,20 );
  write_reg(0xb2,0x02);
  write_reg(0xb1,0x00);
  set_double_reg(0x10,0x28,0x28);
  test_double_reg( 0x20, 0x3a, 0xf2,21 );

  write_reg(0xb3,0x01);
  write_reg(0xb3,0x01);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg(0x12,0xff,0x20);
  set_double_reg(0x14,0xf8,0x28);

  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg(0x10,0x28,0x28);

  write_reg(0xd9,0x80);

  memcpy(registers,switch_on_regs,254);
  write_bulk_registers(registers,0xb3,0);
  write_bulk_registers(registers+0xb3,0x3f,0xb4);
/*  need we a new callibriation ? */
}


/****************************************************************************/
void switch_to_bw(void)
/****************************************************************************/
{
  test_reg( 0x1a,0x04, 30);
  write_reg(0x1a,0x00);
  test_reg( 0xb1,0x00, 31);
  test_reg( 0xb3,0x00, 32);
  test_double_reg( 0x10,0x28,0x3f, 33);
  set_double_reg(0x10,0x18,0x28);
};

/****************************************************************************/
void switch_to_color(void)
/****************************************************************************/
{
  test_reg( 0xb1,0x00, 40);
  test_reg( 0xb3,0x00, 41);
  test_double_reg( 0x10,0x18,0x3b, 42);
  test_reg( 0xb3,0x00, 43);
  set_double_reg(0x10,0x28,0x28);
};

/****************************************************************************/
unsigned int data_ready(unsigned int *size) //Return the number of bytes ready for collection.
/****************************************************************************/
{
    unsigned char command[]={0x90,0x00,0x00,0x03};
    unsigned char data[5];

/*    if ( read_reg(MOVE_START_STOP) & 0x08) // check if the scanner is moving
    {*/
      write_usb(device,0x02,command,4,TIMEOUT);
      read_usb(device,0x81,data,3,TIMEOUT);
      *size = data[2]<<16|data[1]<<8|data[0];
      if(*size > 0xffc0) *size=0xffc0;
#ifdef DEBUG
      printf("Data ready: size= %x\n",*size);
#endif
      if (*size > 0 )
      return(TRUE);
/*    }*/
    else
      return(FALSE);
};

/****************************************************************************/
int read_data(int size,unsigned char *data)
/****************************************************************************/
{
    unsigned char write_command[]={0x91,0x00,0x00,0x00};

//   printf("read_data size %x : %x %x %x\n",size,(size>>16) & 0xff,(size>>8) & 0xff,size & 0xff);
  if(size > 0){
    write_command[3]=size & 0xff;
    write_command[2]=(size>>8) & 0xff;
    write_command[1]=(size>>16) & 0xff;
    write_usb(device,0x02,write_command,4,TIMEOUT);
    read_usb(device,0x81,data,size,TIMEOUT);
  };
  return(0);
};


/****************************************************************************/
void write_cal_bytes( unsigned char b ) {
/****************************************************************************/
   unsigned int i,n;

#ifdef DEBUG
   printf("write cal_bytes %d\n",b);
#endif
   callibration_buffer[0]=0x89;
   callibration_buffer[1]=0x00;
   callibration_buffer[2]=0x00;
   callibration_buffer[3]=0x20;
   callibration_buffer[4]=0;
   n = 2;
   for(i=5;i<36;i++) {
      callibration_buffer[i]   = b;
      callibration_buffer[i+1] = n;
      i++;
      n = n + 2;
   };
   write_usb(device,0x02,callibration_buffer,36,TIMEOUT);
};


/****************************************************************************/
void cal_scanner(void ) {
/****************************************************************************/
  unsigned int i,n;
  unsigned char buffer2[0x818+4];

#ifdef DEBUG
  printf("init scanner...\n");
#endif
  usb_IO_error = write_reg(0xda,0xa7);
  if (usb_IO_error == 0){
/*    printf("no error after first usb_write.\n"); */
    write_reg(MOVE_START_STOP,0x00);
    write_reg(MOVE_START_STOP,0x00);

    init_array(1,TRUE);

    send_my_values(patch_2,myinit_2);

    send_my_values(patch_3,myinit_3);

    set_double_reg( LEDSTATUS, 0x20, 0x28 );
    set_double_reg( LEDSTATUS, 0x28, 0x28 );

    write_reg(MOVE_START_STOP,0x02);
    write_reg(MOVE_START_STOP,0x02);
    write_reg(MOVE_START_STOP,0x00);
    write_reg(MOVE_START_STOP,0x00);
    write_reg(MOVE_START_STOP,0x00);
    write_reg(MOVE_START_STOP,0x00);

    send_my_values(patch_4,myinit_4);

    /**************************************/
#ifdef DEBUG
    printf("scaner calibration...");
#endif
    for (n=0;n<3;n++){
      switch (n){
        case 0: write_reg(0x93,0x06); break;
        case 1: write_reg(0x93,0x02); break;
        case 2: write_reg(0x93,0x01); break;
        default:;
      }
      set_double_reg( 0x91, 0x81, 0x00 );
      callibration_buffer[0]=0x89;
      callibration_buffer[1]=0x00;
      callibration_buffer[2]=0x08;
      callibration_buffer[3]=0x18;
      callibration_buffer[4]=0;
      for(i=5;i<0x818;i++) {
         callibration_buffer[i]=callibration_buffer[i-1]+1;
         if(callibration_buffer[i]==0x61) callibration_buffer[i]=0;
      };
      write_usb(device,0x02,callibration_buffer,0x818+4,TIMEOUT);
      set_double_reg( 0x91, 0x81, 0x00 );
      read_bulk_callibriation_data(buffer2,0x0818);
    };

    init_array(5,TRUE);

    for(i=0;i<5;i++) {
      write_cal_bytes( i );
      set_double_reg( 0x91, 0x00, 0x10 );
      read_bulk_callibriation_data(buffer2,0x20);
    };
    write_reg(LAMPREG,0x80);
    write_reg(LAMPREG,0xad);
    set_double_reg( 0x14, 0x78, 0x28 );
    write_reg(0xda,0xa0);  //05299
    write_reg(0xda,0xa7);  //05352
    write_reg(LAMPREG,0x8d); //05427
    write_reg(LAMPREG,0xad);  //05449
    write_reg(BUTTON_1,0x00);  //05736
    write_reg(BUTTON_2,0x00);  //05811
    write_reg(BUTTON_1,0x00);  //06045
    write_reg(BUTTON_2,0x00);  //06120

    write_reg(0xda,0xa0);  //00423
    write_reg(0xda,0xa7);  //00476
    write_reg(MOVE_START_STOP,0x00); //060101
    write_reg(MOVE_START_STOP,0x00); //060123

    set_double_reg( LEDSTATUS, 0x28, 0x3f );  //060198
    write_reg(LAMPREG,0xad);  //060220

    set_double_reg( LEDSTATUS, 0x20, 0x3f );  //060242

    scannerint = 0x01;  //TRUE
#ifdef DEBUG
    printf("done.\n");
#endif
  }
  else{
    scannerint = 0x00;  //FALSE
#ifdef DEBUG
    printf("fail.\n");
#endif
  };
}


/****************************************************************************/
int park_to_home(void){ //Moves scanner to home position
/****************************************************************************/
#ifdef DEBUG
  printf("park to home...,");
#endif
  if (!scannerint){
#ifdef DEBUG
     printf("I must init the scanner..");
#endif
     cal_scanner(); };

    if (usb_IO_error == 0){

      if(read_reg(MICROSWITCH) & 0x02) {
#ifdef DEBUG
        printf("i'm home\n");
#endif
        return(0); //already there
      } else {
#ifdef DEBUG
      printf("i must mooving\n");
#endif
      init_array(31,TRUE);
/*      send_my_values(patch_move_back,my__move_back); it doesn't work in al cases ! */
      write_reg(0xd3,0x00);
      send_moving_seq();
      while((read_reg(MOVE_START_STOP) & 0x08) && !(read_reg(MICROSWITCH) & 0x02))
      {
/*
         the windows driver is here looking to the buttons.
         here the sequenz :
        80 b3 00 01 <00
        80 25 00 01 <00 button reg
        80 1a 00 01 <00 button reg
        80 b1 00 01 <00
        88 25 00 01 00 reset
        80 1a 00 01 <00
        88 1a 00 01 00 reset
*/
        usleep(10000);
#ifdef DEBUG
        printf(".");
#endif
      }; /*are we there yet? */
      while(read_reg(MOVE_START_STOP)) write_reg(MOVE_START_STOP,0); /*stop it and make sure it stopped!!! */
    };
    cal_scanner();
    return(1);
  }
   else return(0);
};

int num;
/* dump a color buffer in a color PNM */
/****************************************************************************
static void
DumpRGB (int width, int height, unsigned char *data, char *name)
/ ****************************************************************************
{
  FILE *fic;
  char titre[80];
  int y, x;

  if (name == NULL)
    {
      sprintf (titre, "dump%04d.pnm", num);
      num++;
    }
  else
    {
      sprintf (titre, "%s", name);
    }
  fic = fopen (titre, "wb");
  printf ("DumpRGB : P6 %d %d 255\n", width, height);
  fprintf (fic, "P6\n%d %d\n255\n", width, height);
  if (fic == NULL)
    {
      fprintf(stderr,"could not open %s for writing\n", titre);
      return;
    }
  for (y = 0; y < height; y++)
    {
      for (x = 0; x < width; x++)
	{
	  fputc (data[3 * y * width + x * 3], fic);
	  fputc (data[3 * y * width + x * 3 + 1], fic);
	  fputc (data[3 * y * width + x * 3 + 2], fic);
	}
    }
  fclose (fic);
}
*/

/* dump data has received from scanner (red line/green line/blue line)
   to a color pnm file */
/****************************************************************************
static void
DumpRVB (int width, int height, unsigned char *data, char *name)
/ ****************************************************************************
{
  FILE *fic;
  char titre[80];
  int y, x;

  if (name == NULL)
    {
      sprintf (titre, "dump%04d.pnm", num);
      num++;
    }
  else
    {
      sprintf (titre, "%s", name);
    }
  fic = fopen (titre, "wb");
  if (fic == NULL)
    {
      fprintf(stderr,"could not open %s for writing\n", titre);
      return;
    }
  printf ("DumpRVB : P6 %d %d 255\n", width, height);
  fprintf (fic, "P6\n%d %d\n255\n", width, height);
  for (y = 0; y < height; y++)
    {
      for (x = 0; x < width; x++)
	{
	  fputc (data[3 * y * width + 2 * width + x], fic);
	  fputc (data[3 * y * width + width + x], fic);
	  fputc (data[3 * y * width + x], fic);
	}
    }
  fclose (fic);
}
*/

/****************************************************************************/
int wait_for_WarmUp( void ){
/****************************************************************************/
  /* Reads 11 bytes and check the last one */

  /* we must here implement a wait loop for the lamp */
  read_bulk_registers(read_buffer,0x0b,0x84);
/*  printf("Read 0x84 %x , await 00 10 00 00 18 00 00 00 00 20 (60)\n", read_buffer[0xa] &0x60); */
  return( (read_buffer[0xa] & 0x60) == 0x60 );
};



/****************************************************************************/
int move_pos(unsigned int pos,unsigned char speed, unsigned char stepsize, unsigned char move_forward)
/****************************************************************************/
{
  unsigned char myregs[255];

  printf("have to move to 0x%x, that is %d \n",pos,(pos * stepsize));
  if ( (pos * stepsize)  > 7350 ){
    fprintf(stderr,"pos %d outside moving region\n",pos);
    pos = 7350 / stepsize;
    printf("Set pos to %x %d\n",pos,pos);
  };

  /* if scanner is moving, we must waiting */
  while((read_reg(MOVE_START_STOP) & 0x08) ){ printf("!");usleep(1000000);}; //are we there yet?

  if (!scannerint){
#ifdef DEBUG
     printf("I must init the scanner..");
#endif
     cal_scanner(); };

  printf("Move ");
  if (move_forward){printf("forward");} else {printf("backward");};
  printf(" to %x with stepsize %x\n",pos,stepsize);
/*  read_all_registers(myregs);*/

  memcpy(myregs,myinit33,254);    //00049277 pv75dpi_g23

/*
  set_double_reg(0x10,0x20,0x1b);
  memcpy(myregs,myinit33,254);*/

  printf("current regs :reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x, reg_3A 0x%x, reg_61/60 0x%x%x, reg_63/62 0x%x%x, reg_65 0x%x, reg_E1 0x%x \n",
          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36],myregs[0x3a],myregs[0x61],myregs[0x60],myregs[0x63],myregs[0x62],myregs[0x65],myregs[0xe1]);

  /* bit 6 direction of movement */
//  if (move_forward){
//    myregs[0x36] = 0x2c;} /*myregs[0x36] | 0x08;}*/
//  else{
    myregs[0x36] = move_forward; //0x24;} /*myregs[0x36] & 0xf7;*/

  myregs[0x33] = 0x81; /* fast move */
/*  myregs[0x33] = 0x80 & stepsize;  fast move */
  myregs[0x3a] = 0x1b;
  myregs[0x65] = 0x00;

//  myregs[0x60] = (pos) & 0xff;
//  myregs[0x61] = ((pos) >> 8) & 0xff;
  myregs[0x60] = 0x12;
  myregs[0x61] = 0;
  myregs[0x62]=(pos+1) & 0xff;
  myregs[0x63]=((pos+1) >> 8) & 0xff;
//  myregs[0x62] = (pos+1) & 0xff;
//  myregs[0x63] = ((pos+1) >> 8) & 0xff;
  myregs[0xe1] = stepsize;
  printf("patched regs :reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x, reg_3A 0x%x, reg_61/60 0x%x%x, reg_63/62 0x%x%x, reg_65 0x%x, reg_E1 0x%x \n",
          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36],myregs[0x3a],myregs[0x61],myregs[0x60],myregs[0x63],myregs[0x62],myregs[0x65],myregs[0xe1]);

  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  send_moving_seq();
  /* wait for start of movement */
  while((read_reg(MOVE_START_STOP) & 0x08) /*&& !(read_reg(MICROSWITCH) & 0x02)*/)
  {
    usleep(10000);
    printf(".");
  };
//  while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?
  printf("done\n");

  return(0);
};


/****************************************************************************/
int init_b3_12_14_10(unsigned int resolution, unsigned int init_number)
/****************************************************************************/
{
  char init[]={0x88,0xb3,0x00,0x01,0x01,
               0x88,0xb3,0x00,0x01,0x01,
               0x88,0xb3,0x00,0x01,0x00,
               0x88,0xb3,0x00,0x01,0x00,
               0x88,0x12,0x00,0x02,0xff,0x20,
               0x88,0x14,0x00,0x02,0xf8,0x28,
               0x88,0xb3,0x00,0x01,0x00,
               0x88,0xb3,0x00,0x01,0x00,
               0x88,0x10,0x00,0x02,0x20,0x3f,
               0x88,0xd9,0x00,0x01,0x80};

  switch(resolution) {

    case R_150_16:
      init[46] = 0x20;
      init[47] = 0x3f;
      break;

    case R_75_g:
      switch(init_number) {
        case 1:
          init[46] = 0x28;
          init[47] = 0x28;
          break;
        case 2:
          init[46] = 0x28;
          init[47] = 0x3f;
          break;
#ifdef DEBUG
        default: printf("init_b3_12_14_10 R_75_g: Error, wrong init_number %x.\n",init_number);
#endif
      };
      break;

#ifdef DEBUG
    default: printf("init_b3_12_14_10 : Error, wrong resolution %x.\n",resolution);
#endif
  };

  write_usb(device,0x02,init,53,TIMEOUT);
  return(0);
}


/****************************************************************************/
int init_bevore_scan(unsigned int resolution)
/* plugin+150dpi_16.log :
   new 11.01.2003 */
/****************************************************************************/
{
  unsigned char myregs[255];
  {
    test_reg( 0xda,0xa7,110);          //00847169
    write_reg(0xda,0xa0);              //00847185
    switch(resolution) {
      case R_150_16:
        test_double_reg(0x10,0x20,0x3f,111);
        break;
      case R_75_g:
        test_double_reg(0x10,0x68,0x3f,111); //00016994
        break;
#ifdef DEBUG
      default: printf("init_bevore_scan 1: Error, wrong resolution %x.\n",resolution);
#endif
    };
    write_reg(0xda,0xa7);              //00847201
    test_reg( 0xda,0xa7,112);          //00847208
    test_reg( 0xb3,0x00,1121);          //00847224

    init_b3_12_14_10(resolution,1);   //00847240

    switch(resolution) {
      case R_150_16:
        memcpy(myregs,pv150dpi_20,254);   //00847253
        break;
      case R_75_g:
        memcpy(myregs,pv75dpi_g20,254);   //00017062
        break;
#ifdef DEBUG
      default: printf("init_bevore_scan 2: Error, wrong resolution %x.\n",resolution);
#endif
    };
    write_bulk_registers(myregs,0xb3,0);
    write_bulk_registers(myregs+0xb3,0x40,0xb4);     //15.01.03 JoHu

    switch(resolution) {
      case R_150_16:
    /* this sequenz is the same by scan_150dpi_16_color and scan_300dpi_colorn */
        write_reg(0xda,0xa7);
        write_reg(0xd9,0x8d);
        write_reg(0xd9,0xad);
        write_reg(0xb3,0x00);
        write_reg(0xb3,0x00);
        test_double_reg(0x10,0x20,0x3f,105);
        set_double_reg( 0x10,0x20,0x3f);
        write_reg(0xd9,0xad);
        test_reg( 0xb3,0x00,106);
        test_reg( 0x1d,0x22,107);
        test_reg( 0xb3,0x00,108);
        test_reg( 0xda,0xa7,109);
        write_reg(0xda,0xa0);
        test_double_reg(0x10,0x20,0x3f,110);
        write_reg(0xda,0xa7);
        test_reg( 0xda,0xa7,111);
        test_reg( 0xb3,0x00,112);
        test_reg( 0xda,0xa7,113);
        write_reg(0xda,0xa0);
        test_double_reg(0x10,0x20,0x3f,114);
        write_reg(0xda,0xa7);
        test_reg( 0xda,0xa7,115);
        test_reg( 0xb3,0x00,116);
        init_b3_12_14_10(resolution,2);   //00017187 by 75_g
        memcpy(myregs,pv150dpi_21,254);     //00847565
        break;
      case R_75_g:
        write_reg(0xda,0xa0);  //00017106
        write_reg(0xd9,0x8d);
        write_reg(0xd9,0xa2);
        write_reg(0xb3,0x00);
        write_reg(0xb3,0x00);
        test_double_reg(0x10,0x28,0x3f,105);
        set_double_reg( 0x10,0x28,0x3f);
        write_reg(0xd9,0xa2);
        init_b3_12_14_10(resolution,2);   //00017187 by 75_g
        memcpy(myregs,pv150dpi_21,254);     //00017200
        break;
#ifdef DEBUG
      default: printf("init_bevore_scan 3: Error, wrong resolution %x.\n",resolution);
#endif
    };

    write_bulk_registers(myregs,0xb3,0);
    write_bulk_registers(myregs+0xb3,0x40,0xb4);  //15.01.03 JoHu

    switch(resolution) {
      case R_150_16:
        write_reg(0xda,0xa7);              //00847609
        write_reg(0xd9,0x8d);
        write_reg(0xd9,0xad);
        write_reg(0xb3,0x00);
        write_reg(0xb3,0x00);
        while (!test_double_reg(0x10,0x20,0x3b,117)) //00847660
        {
          set_double_reg( 0x10,0x20,0x3b);
        };
        write_reg(0xd9,0xad);
//       } while (!test_double_reg(0x10,0x20,0x3b,118));
        break;
      case R_75_g:
        write_reg(0xda,0xa7);              //00017244
        write_reg(0xd9,0x82);
        write_reg(0xd9,0xad);
        write_reg(0xb3,0x00);
        write_reg(0xb3,0x00);
        test_double_reg(0x10,0x28,0x3f,117); //00017279
        set_double_reg( 0x10,0x28,0x3f);
        write_reg(0xd9,0xad);

        test_reg( 0xb3,0x00,106); //00017309
        test_reg( 0x1d,0x22,107);
        test_reg( 0xb3,0x00,108);
        test_reg( 0xda,0xa7,109);
        write_reg(0xda,0xa0);
        test_double_reg(0x10,0x28,0x3f,110);
        write_reg(0xda,0xa7);
        test_reg( 0xda,0xa7,111);
        test_reg( 0xb3,0x00,112);
        test_reg( 0xda,0xa7,113);
        write_reg(0xda,0xa0);
        test_double_reg(0x10,0x28,0x3f,114);
        write_reg(0xda,0xa7);
        test_reg( 0xda,0xa7,115);
        test_reg( 0xb3,0x00,116);

        init_b3_12_14_10(resolution,2);   //00017499 by 75_g

        memcpy(myregs,pv150dpi_22,254);     //00017512
        write_bulk_registers(myregs,0xb3,0);
        write_bulk_registers(myregs+0xb3,0x40,0xb4);  //15.01.03 JoHu

        write_reg(0xda,0xa7);              //00017556
        write_reg(0xd9,0x8d);
        write_reg(0xd9,0xad);
        write_reg(0xb3,0x00);
        write_reg(0xb3,0x00);

        test_double_reg(0x10,0x28,0x3f,117); //00017591
        set_double_reg(0x10,0x28,0x3f);    //
        write_reg(0xd9,0xad);

        break;
#ifdef DEBUG
      default: printf("init_bevore_scan 4: Error, wrong resolution %x.\n",resolution);
#endif
    };

  }; // while (!test_double_reg(0x10,0x20,0x3b,118));
  return(0);
};

/****************************************************************************/
int scan_pv_color(int length,int x1,int x2/*, int x3,int x4,char *filename*/)
/* pv.log : previev 21,59cm X 2,3cm by ? dpi, ? collors */
/****************************************************************************/
{
  unsigned char myregs[255];
  unsigned int i /*,n*/;
  unsigned int size;
  unsigned char *data /*, stepsize*/;
  unsigned int data_count;
  unsigned int max_size;
  FILE *FD; //test pnm file
  clock_t start, end;
  clock_t elapsed;

  int count = 5; //5   37;

  move_pos(150,0x0e,01,0x29);
/*return(TRUE);**/
  /* Some sanity checks: */
  if(x1<=0) {
#ifdef DEBUG
    fprintf(stderr,"x1 outside scanning region\n");
#endif
    exit(-1);};
  if(!(data=malloc(0xffffff))) {
#ifdef DEBUG
    fprintf(stderr,"Cant allocate memory");
#endif
    exit(-1);};


  write_reg(0xda,0xa7);    //00032743
  test_reg( 0xda, 0xa7, 1);  //00032765
  test_reg( 0xb3, 0x00, 2 );  //00032818
  test_reg( 0xda, 0xa7, 3 );  //00032871
  write_reg(0xda,0xa0);    //00032924
  test_double_reg( 0x10, 0x28, 0x3f, 4 );
  write_reg(0xda,0xa7);    //00032977
  test_reg( 0xda, 0xa7, 5 );  //00032999
  test_reg( 0xb3, 0x00, 6 );  //00033052
  write_reg(0xb3,0x01);    //00033105
  write_reg(0xb3,0x01);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg( 0x12, 0xff, 0x20 );
  set_double_reg( 0x14, 0xf8, 0x28 );
  write_reg(0xb3,0x00);    //00033109
  write_reg(0xb3,0x00);
  set_double_reg( 0x10, 0x28, 0x3f );
  write_reg(0xd9,0x80);    //00033111


  while ( !wait_for_WarmUp() )
#ifdef DEBUG
    printf(".");
#endif
    /*sleep(1)*/;

  memcpy(myregs,pvinit10,254);
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xda,0xa7);                    //00033207
  write_reg(0xd9,0x8d);                    //00033229
  write_reg(0xd9,0xad);                    //00033251
  write_reg(0xb3,0x00);                    //00033273
  write_reg(0xb3,0x00);                    //00033295
  test_double_reg( 0x10, 0x28, 0x3f, 7 );  //00033317
  set_double_reg( 0x10, 0x28, 0x3f );      //00033370
  test_double_reg( 0x10, 0x28, 0x3f, 8 );  //new added
  write_reg(0xd9,0xad);                    //00033392

  FD=fopen("preview.pnm","w");
#ifdef DEBUG
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length*count,x1,x2);
#endif  
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length*count);

  i = 0;
  while( i < count ){

#ifdef DEBUG
  printf("scan count %d ",i); 
#endif
    set_double_reg( 0x10, 0x20, 0x3f );       //00033414

    memcpy(myregs,myinit33,254);

/*********************************************************************************/
    myregs[0x60]= 1;
    myregs[0x61]= 0;
    myregs[0x62]= (length+1) & 0xff;
    myregs[0x63]= ((length+1) >> 8) & 0xff;

    myregs[0x66]=x1 & 0xff;
    myregs[0x67]=(x1 >> 8) & 0xff;
    myregs[0x6c]=x2 & 0xff;
    myregs[0x6d]=(x2 >> 8) & 0xff;

    write_bulk_registers(myregs,0xb3,0);
    write_bulk_registers(myregs+0xb3,0x3f,0xb4);

/*    DumpHex(myregs, 0xf0, 16);*/

    write_reg(0xd3,0x03);                   //00033510

    start = clock();
    max_size = 0;
    send_moving_seq();                      //00033532

    while(!data_ready((unsigned int*) &size)) sleep(1);

    while(/*size=*/data_ready((unsigned int*) &size)) {              //00033664
      read_data(size,data);
      data_count=0;
      if (size >= max_size) max_size = size;
      printf("size=%x\n",size);
      while(size--) fprintf(FD,"%c",data[data_count++]);
    };

    while((read_reg(MOVE_START_STOP) & 0x08) /*&& !(read_reg(MICROSWITCH) & 0x02)*/)
    {
      usleep(10000);
      printf(".");
    };
    printf("scanner is stoped. Max size = %x\n",max_size);
    /* Do the work. */
    end = clock();
    elapsed = (end - start)/* / CLOCKS_PER_SEC*/;
/*    printf("transfertime = start %x - end %x = %x (%x)\n",start, end, elapsed,CLOCKS_PER_SEC);*/
    i++;
  };

  fclose(FD);
  free(data);
  printf("done\n");
  return(0);
};



/****************************************************************************/
int scan_600dpi_color(int length,int x1,int x2)
/****************************************************************************/
{
  unsigned char myregs[255];
  unsigned int i /*,n*/;
  unsigned int size;
  unsigned char *data /*, stepsize*/;
  unsigned int data_count;
  FILE *FD; //test pnm file
  int count = 2; //37;


  /* Some sanity checks: */
  if(x1<=0) {fprintf(stderr,"x1 outside scanning region\n"); exit(-1);};
  if(!(data=malloc(0xffffff))) {fprintf(stderr,"Cant allocate memory");exit(-1);};

// 00827315  68.14216400   88 b3 00 01 08 move back, than wait loop

  test_reg( 0xda,0xa7,7);           // 00836284
  write_reg(0xda,0xa0);             // 00836300
  if (!test_double_reg( 0x10, 0x20,0x3f, 8))
    set_double_reg( 0x10,0x20,0x3f);

  test_reg( 0xda,0xa7,9);           //00836323
  write_reg(0xb3,0x01);             // 00836355

  write_reg(0xb3,0x01);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg( 0x12, 0xff,0x20 );
  set_double_reg( 0x14, 0xf8,0x28 );
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg( 0x10, 0x20,0x3f );
  write_reg(0xd9,0x80);

/*  printf("Start init 600dpi 1....................\n"); */

  memcpy(myregs,pv600dpi_11,254);   //  00836368
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xda,0xa7);             // 00836412
  write_reg(0xd9,0x8d);             // 00836419
  write_reg(0xd9,0xad);             // 00836426
  write_reg(0xb3,0x00);             // 00836433
  write_reg(0xb3,0x00);             // 00836440
  if (!test_double_reg( 0x10, 0x20,0x3b, 10)) // 00836447
    set_double_reg( 0x10,0x20,0x3b);  //
  set_double_reg( 0x10,0x20,0x3b);  // 00836463
  write_reg(0xd9,0xad);             // 00836470


  test_reg( 0xb3,0x00,11); // 00836477
  test_reg( 0x1d,0x22,12); // 00836493
  test_reg( 0xb3,0x00,13); // 00836509
  test_reg( 0xda,0xa7,14); // 00836525
  write_reg(0xda,0xa0); // 00836541
  test_double_reg( 0x10, 0x20,0x3f, 15);
  write_reg(0xda,0xa7); // 00836557
  test_reg( 0xda,0xa7,16); // 00836564
  test_reg( 0xb3,0x00,17); // 00836580
  test_reg( 0xda,0xa7,18); // 00836596
  write_reg(0xda,0xa0); // 00836612
  test_double_reg( 0x10, 0x20,0x3f, 19);
  write_reg(0xda,0xa7); // 00836628
  test_reg( 0xda,0xa7,20); // 00836635
  test_reg( 0xb3,0x00,21); // 00836651


  write_reg(0xb3,0x01); //00836667
  write_reg(0xb3,0x01);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg( 0x12, 0xff,0x20 );
  set_double_reg( 0x14, 0xf8,0x28 );
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg( 0x10, 0x20,0x3b );
  write_reg(0xd9,0x80);

/*  printf("Start init 600dpi 2....................\n"); */

  memcpy(myregs,pv600dpi_12,254);   //00836680
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xda,0xa7);             // 00836724
  write_reg(0xd9,0x8d);             // 00836731
  write_reg(0xd9,0xad);             // 00836738
  write_reg(0xb3,0x00);             // 00836745
  write_reg(0xb3,0x00);             // 00836752
  if (!test_double_reg( 0x10, 0x20,0x3f, 22)) //00836759
    ;//set_double_reg( 0x10,0x20,0x3b);  //
  set_double_reg( 0x10,0x20,0x3f);  // 00836775
  write_reg(0xd9,0xad);             // 00836782

/*  printf("Start scanning 600dpi 3....................\n"); */

/*
  memcpy(myregs,pv600dpi_13,254);   //00836796
  length = myregs[0x62] + 1;
  length = length + ((myregs[0x63] <<8) & 0xFF00);

  myregs[0x66]=x1 & 0xff;
  myregs[0x67]=(x1 >> 8) & 0xff;
  myregs[0x6c]=x2 & 0xff;
  myregs[0x6d]=(x2 >> 8) & 0xff;

  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
*/

  FD=fopen("test600dpi.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,/*406*/length );

  i = 0;
  while( i < count ){

/*  printf("scan count %d ",i); */

  set_double_reg( 0x10,0x20,0x3f);  // 00836789
  memcpy(myregs,pv600dpi_13,254);   //00836796

/***************************************************************************/
  myregs[0x60]=(length) & 0xff;
  myregs[0x61]=((length) >> 8) & 0xff;
  myregs[0x62]=(length+1) & 0xff;
  myregs[0x63]=((length+1) >> 8) & 0xff;

  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
  length = myregs[0x62] + 1;
  length = length + ((myregs[0x63] <<8) & 0xFF00);
#ifdef DEBUG
  printf("scan_600dpi_color.pv600dpi_13.........................................................\n");
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("\n");
  printf("reg_10 0x%x, reg_11 0x%x, reg_65 0x%x, reg_e1 0x%x\n",myregs[0x10],myregs[0x11],myregs[0x65],myregs[0xe1]);
#endif
/***************************************************************************/


  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xd3,0x03); //00836840
  send_moving_seq(); //

/*  while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; /are we there yet?*/


  while(!data_ready((unsigned int*) &size)) sleep(1);
  while(data_ready((unsigned int*) &size)) {
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };

//  DumpRVB ((x2-x1)/3, length, data, "Test1.pnm");
//  DumpRGB ((x2-x1)/3, length, data, "Test2.pnm");
//  printf("\n");
  i++;
  };


/* move back ?
  set_double_reg( 0x10,0x20,0x3f);  // 00871626
  memcpy(myregs,pv600dpi_14,254);   //00871633

  write_reg(0xd3,0x03);               //00871677
  send_moving_seq();

  printf("start moving...");
  while((read_reg(MOVE_START_STOP) & 0x08) && !(read_reg(MICROSWITCH) & 0x02)) usleep(1000000); //00857736 /00860072  127.62494960   80 b3 00 01 <00
  printf("done\n");
*/
#ifdef DEBUG
  printf("OK\n");
#endif
  fclose(FD);
  free(data);
  return(0);
};


/****************************************************************************/
int scan_300dpi_colorn(int length,int x1,int x2)
// plugin+300dpi.log : previev 21,59cm X 2,3cm by ? dpi, ? collors
// new 07.01.2003
/****************************************************************************/
{
  unsigned char myregs[255];
/*  unsigned int i,n; */
  unsigned int size;
  unsigned char *data/*, stepsize*/;
  unsigned int data_count;
  FILE *FD; //test pnm file

/*  int count = 1;*/

  //Some sanity checks:
  if(x1<=0) {fprintf(stderr,"x1 outside scanning region\n"); exit(-1);};
  if(!(data=malloc(0xffffff))) {fprintf(stderr,"Cant allocate memory");exit(-1);};



  write_reg(0xd9,0x8d);              //00823394
  write_reg(0xd9,0xad);              //00823401
  test_reg( 0xb3,0x00,100);          //00823408
  test_reg( 0xda,0xa7,101);          //00823424
  write_reg(0xda,0xa0);              //00823440
  test_double_reg(0x10,0x20,0x3f,102);
  write_reg(0xda,0xa7);              //00823456
  test_reg( 0xda,0xa7,103);          //00823463
  test_reg( 0xb3,0x00,104);          //00823479

  write_reg(0xb3,0x01);              //00823495
  write_reg(0xb3,0x01);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg(0x12,0xff,0x20);
  set_double_reg(0x14,0xf8,0x28);
  write_reg(0xb3,0x00);              //00823499
  write_reg(0xb3,0x00);
  set_double_reg(0x10,0x20,0x3b);
  write_reg(0xd9,0x80);              //00823501

  memcpy(myregs,pv300dpi_20,254);    //00823508
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xda,0xa7);              //00823552
  write_reg(0xd9,0x8d);              //00823559
  write_reg(0xd9,0xad);              //00823566
  write_reg(0xb3,0x00);              //00823573
  write_reg(0xb3,0x00);              //00823580
  test_double_reg(0x10,0x20,0x3f,105);//00823587
  set_double_reg( 0x10,0x20,0x3f);   //00823603
  write_reg(0xd9,0xad);              //00823610
  test_reg( 0xb3,0x00,106);          //00823617
  test_reg( 0x1d,0x22,107);          //00823633
  test_reg( 0xb3,0x00,108);          //00823649
  test_reg( 0xda,0xa7,109);          //00823665

  write_reg(0xda,0xa0);              // 00823681
  test_double_reg(0x10,0x20,0x3f,110);//
  write_reg(0xda,0xa7);              //00823697
  test_reg( 0xda,0xa7,111);          //00823704
  test_reg( 0xb3,0x00,112);          //00823720
  test_reg( 0xda,0xa7,113);          //00823736
  write_reg(0xda,0xa0);              //00823752
  test_double_reg(0x10,0x20,0x3f,114);
  write_reg(0xda,0xa7);              //00823768
  test_reg( 0xda,0xa7,115);          //00823775
  test_reg( 0xb3,0x00,116);          //00823791
  write_reg(0xb3,0x01);              //00823807
  write_reg(0xb3,0x01);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);

  set_double_reg(0x12,0xff,0x20);
  set_double_reg(0x14,0xf8,0x28);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  set_double_reg(0x10,0x20,0x3f);
  write_reg(0xd9,0x80);

  memcpy(myregs,pv300dpi_21,254);    //00823820
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xda,0xa7);              //00823864
  write_reg(0xd9,0x8d);              //00823871
  write_reg(0xd9,0xad);              //00823878
  write_reg(0xb3,0x00);              //00823885
  write_reg(0xb3,0x00);              //00823892
  test_double_reg(0x10,0x20,0x3b,117);//00823899
  set_double_reg(0x10,0x20,0x3b);
  test_double_reg(0x10,0x20,0x3b,118);//00823929
  write_reg(0xd9,0xad);              //00823922
  set_double_reg(0x10,0x20,0x3b);

/***** start of 300dpi scanning (moving)******************/

  memcpy(myregs,pv300dpi_22,254);    //00823936
/***************************************************************************/


  myregs[0x62]=(length+1) & 0xff;
  myregs[0x63]=((length+1) >> 8) & 0xff;

  length = myregs[0x62] + 1;
  length = length + ((myregs[0x63] <<8) & 0xFF00);
  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);

  FD=fopen("Preview1.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);
#ifdef DEBUG
  printf("Preview...(scan_300dpi_colorn)......................................................\n");
  printf("\n");
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("\n");
  printf("reg_10/11 0x%x%x, reg_65 0x%x, reg_e1 0x%x\n",myregs[0x10],myregs[0x11],myregs[0x65],myregs[0xe1]);
#endif
/***************************************************************************/

  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xd3,0x03);              //00823980
  send_moving_seq();                 //00823987

  while(!data_ready((unsigned int*) &size)) sleep(1);     //00824029
  while(data_ready((unsigned int*) &size)) {
//00824568  125.23189680   ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };

//  DumpRVB (x2-x1, length, data, "Test1.pnm");
//  DumpRGB (x2-x1, length, data, "Test2.pnm");

  fclose(FD);
//usleep(10000000);
/**** start of moving *****************/
/*
//00857631  ???? < 00
  set_double_reg(0x10,0x20,0x3b);     //00857636
  memcpy(myregs,pv300dpi_23,254);     //00857643
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x03);               //00857687
  send_moving_seq();                  //00857694

  printf("start moving...");
  while((read_reg(MOVE_START_STOP) & 0x08) && !(read_reg(MICROSWITCH) & 0x02)) usleep(1000000); //00857736 /00860072  127.62494960   80 b3 00 01 <00
  printf("done\n");
*/
 /** here is a error, the scanner doesn't move *****/

  set_double_reg(0x10,0x20,0x3b);     //00860088
  memcpy(myregs,pv300dpi_24,254);     //00860095

/***************************************************************************/
#ifdef DEBUG
  printf("Scan 1 .pv300dpi_24........................................................\n");
#endif
  length = myregs[0x62] + 1;
  length = length + ((myregs[0x63] <<8) & 0xFF00);
//  myregs[0x60]=(length) & 0xff;
//  myregs[0x61]=((length) >> 8) & 0xff;
//  myregs[0x62]=(length+1) & 0xff;
//  myregs[0x63]=((length+1) >> 8) & 0xff;

  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
#ifdef DEBUG
  printf("\n");
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("\n");
  printf("reg_10/11 0x%x%x, reg_65 0x%x, reg_e1 0x%x\n",myregs[0x10],myregs[0x11],myregs[0x65],myregs[0xe1]);
  printf("P5 %x %x 255\n",x2-x1,length);
#endif
//  test_double_reg(0x10,0x20,0x3b,119);//00823929




  /* here is my problem, the scaner doesn't move forward for scan  */
  /* the init sequenze is OK. */



  FD=fopen("test300dpi1.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);

/***************************************************************************/

  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x17);               //00860139
  send_moving_seq();                  //00860146

//    while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?

  while(!data_ready((unsigned int*) &size)) sleep(1);     //00860188
  while(data_ready((unsigned int*) &size)) {
//00860247  127.76390160   16 16 15 16 16 16 16 16 15 16 16 15 16 17 15 16
    read_data(size,data);
    data_count=0;
    while(size--) {fprintf(FD,"%c",data[data_count++]);};
  };

//  DumpRVB (x2-x1, length/3, data, "Test3.pnm");
//  DumpRGB (x2-x1, length/3, data, "Test4.pnm");

  fclose(FD);


/***************************************************************************/
#ifdef DEBUG
  printf("Scan 2 ..pv300dpi_25.......................................................\n");
#endif
  set_double_reg(0x10,0x20,0x3b);     //00860587
  memcpy(myregs,pv300dpi_25,254);     //00860594

  length = myregs[0x62] + 1;
  length = length + ((myregs[0x63] <<8) & 0xFF00);

//  myregs[0x62]=(length+1) & 0xff;
//  myregs[0x63]=((length+1) >> 8) & 0xff;

  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);




  /* here is my problem, the scaner doesn't move forward for scan  */
  /* the init sequenze is OK. */



#ifdef DEBUG
  printf("\n");
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("\n");
  printf("reg_10/11 0x%x%x, reg_65 0x%x, reg_e1 0x%x\n",myregs[0x10],myregs[0x11],myregs[0x65],myregs[0xe1]);
  printf("P5 %x %x 255\n",x2-x1,length);
#endif
  FD=fopen("test300dpi2.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);
/***************************************************************************/

  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x17);               //00860638
  send_moving_seq();                  //00860645

    while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?


  while(!data_ready((unsigned int*) &size)) sleep(1);     //00860687
  while(data_ready((unsigned int*) &size)) {
//00862403  128.08840480   02 03 01 02 02 01 02 02 01 02 02 01 02 02 01 01
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };

//  DumpRVB (x2-x1, length, data, "Test5.pnm");
//  DumpRGB (x2-x1, length, data, "Test6.pnm");

  fclose(FD);
/**/
  free(data);
  return(0);
};

/****************************************************************************/
int scan_150dpi_16_color(int length,int x1,int x2)
// plugin+150dpi_16.log :
// new 11.01.2003
/****************************************************************************/
{
  unsigned char myregs[255];
  unsigned int size;
  unsigned char *data/*, stepsize*/;
  unsigned int data_count;
  FILE *FD; //test pnm file


  //Some sanity checks:
  if(x1<=0) {fprintf(stderr,"x1 outside scanning region\n"); exit(-1);};
  if(!(data=malloc(0xffffff))) {fprintf(stderr,"Cant allocate memory");exit(-1);};

  init_bevore_scan(R_150_16);
  while ( !wait_for_WarmUp() ) printf("."); /*sleep(1);*/

  memcpy(myregs,pv150dpi_22,254);    //00847681

/***************************************************************************/
#ifdef DEBUG
  printf("1. scan with 150 dpi 16 color (warm up).> test150dpi16_warmup.pnm......................\n");
#endif
  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
#ifdef DEBUG
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x\\n",
          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36]);
  printf("\n");
#endif
  FD=fopen("test150dpi16_warmup.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);
/***************************************************************************/

  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x03);               //00847725
  send_moving_seq();

  while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?

  while(data_ready((unsigned int*) &size)) {
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };
  
  fclose(FD);

#ifdef DEBUGM
  set_double_reg(0x10,0x20,0x3b);    //00884116
  memcpy(myregs,pv150dpi_23,254);    //00884123
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x03);               //00884167
  send_moving_seq();
#endif

#ifdef DEBUG
  printf("start moving back...");
#endif
  while((read_reg(MOVE_START_STOP) & 0x08) && !(read_reg(MICROSWITCH) & 0x02)) usleep(1000000); //00857736 /00860072  127.62494960   80 b3 00 01 <00
#ifdef DEBUG
  printf("done\n");
#endif
  init_bevore_scan(R_150_16);

  set_double_reg(0x10,0x20,0x3b);    //00886600
  memcpy(myregs,pv150dpi_24,254);    //00886607

/***************************************************************************/
#ifdef DEBUG
  printf("1. scan with 150 dpi 16 color.> test150dpi16_1.pnm........................................\n");
#endif

//  myregs[0x62]=(length+1) & 0xff;
//  myregs[0x63]=((length+1) >> 8) & 0xff;
#ifdef DEBUGM
  myregs[0x60]= 1;
  myregs[0x61]= 0;
  myregs[0x60]= (x3) & 0xff;
  myregs[0x61]= ((x3) >> 8) & 0xff;
  myregs[0x62]= (x3+1) & 0xff;
  myregs[0x63]= ((x3+1) >> 8) & 0xff;
#endif

  length = myregs[0x62] + 1;
  length = length + ((myregs[0x63] <<8) & 0xFF00);
  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
#ifdef DEBUG
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
#endif

#ifdef DEBUG
  printf("reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x, reg_3A 0x%x, reg_60/61 0x%x%x, reg_62/63 0x%x%x, reg_65 0x%x \n",
          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36],myregs[0x3a],myregs[0x61],myregs[0x60],myregs[0x63],myregs[0x62],myregs[0x65]);
#endif

//  myregs[0x33] = 0x83;
//  myregs[0x36] = 0x2c /*myregs[0x36]  | 0x80*/;    //15.01.03 JoHu

#ifdef DEBUG1
  printf("reg_10 0x%x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x, reg_3A 0x%x, reg_60/61 0x%x%x, reg_62/63 0x%x%x, reg_65 0x%x \n",
          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36],myregs[0x3a],myregs[0x61],myregs[0x60],myregs[0x63],myregs[0x62],myregs[0x65]);
#endif
#ifdef DEBUG
  DumpHex(myregs, 0xf0, 16);
#endif

  FD=fopen("test150dpi16_1.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);
/***************************************************************************/
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  /* here is my problem, the scaner doesn't move forward for scan  */
  /* the init sequenze is OK. */
  /* The reg MOVE_START_STOP (0xb3) goes to 0x08 for one or two times,
     but the scanner doesn' move. I see this problem by all resolutions
     at the moment */
  {
    write_reg(0xd3,0x0e);               //00886651
    send_moving_seq();
    while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?

  } /*while (test_reg( 0xb3,0x08,120));*/

  while(data_ready((unsigned int*) &size)) {
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };
  while(data_ready((unsigned int*) &size)) {
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };
  
//  DumpRVB (x2-x1, length, data, "Test5.pnm");
//  DumpRGB (x2-x1, length, data, "Test6.pnm");

  fclose(FD);
/*

  set_double_reg( 0x10,0x20,0x3b);
  memcpy(myregs,pv150dpi_25,254);    //00887079

***************************************************************************
  printf("2. scan with 150 dpi 16 color.........................................\n");
  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x\n",myregs[0x10],myregs[0x11],myregs[0xe1]);
  printf("\n");

  FD=fopen("test150dpi16_2.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);
***************************************************************************
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x0e);               //00887123
  send_moving_seq();

  while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?

  while(data_ready((unsigned int*) &size)) {  //
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };
//  DumpRVB (x2-x1, length, data, "Test5.pnm");
//  DumpRGB (x2-x1, length, data, "Test6.pnm");

  fclose(FD);
**

  set_double_reg( 0x10,0x20,0x3b);  //00889567
  memcpy(myregs,pv150dpi_26,254);    //00889574

***************************************************************************
  printf("3. scan with 150 dpi 16 color.........................................\n");
  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x\n",myregs[0x10],myregs[0x11],myregs[0xe1]);
  printf("\n");

  FD=fopen("test150dpi16_3.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);
***************************************************************************
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x0e);               //00889618
  send_moving_seq();

  while((read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?

  while(data_ready((unsigned int*) &size)) {  //
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };
//  DumpRVB (x2-x1, length, data, "Test5.pnm");
//  DumpRGB (x2-x1, length, data, "Test6.pnm");

  fclose(FD);
**/
  free(data);
  return(0);
};

/****************************************************************************/
int scan_75dpi_gray(int length,int x1,int x2/*, int x3,int x4,char *filename*/)
// plugin+75dpi_gray.log :
// new 19.01.2003
/****************************************************************************/
{
  unsigned char myregs[255];
#ifndef REG_EDIT
  unsigned int size;
  unsigned int data_count;
#endif

  //Some sanity checks:
  if(x1<=0) {fprintf(stderr,"x1 outside scanning region\n"); exit(-1);};
  if(!(data=malloc(0xffffff))) {fprintf(stderr,"Cant allocate memory");exit(-1);};

#ifdef TEST
  printf("0. init for 75dpi gray warm up scan *****************************************************\n");

  init_bevore_scan(R_75_g);

/* now implemented in init_bevore_scan(R_75_g);
00016978  26.16361840   80 da 00 01 <a7
00016994  26.16743200   88 da 00 01 a0 80 10 00 02 <68 3f  // here comes a wrong feed back
00017010  26.17123760   88 da 00 01 a7
00017017  26.17314320   80 da 00 01 <a7
00017033  26.19147920   80 b3 00 01 <00

00017049
88 b3 00 01 01
88 b3 00 01 01
88 b3 00 01 00
88 b3 00 01 00
88 12 00 02 ff 20
88 14 00 02 f8 28
88 b3 00 01 00
88 b3 00 01 00
88 10 00 02 28 28
88 d9 00 01 80

00017062
 88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70
 00 00 00 00 28 28 ff 20 f8 28 07 00 ff 00 00 00
 00 20 00 00 3a f2 00 ff ff 00 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00
 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
 00 00 00 00 00 00 e1 14 18 15 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 ff
 3f 80 68 00 00 00 00 01 0e 00 00 00 00 00 00 00
 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00
 00 00 00 00 00 00 02
 88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00
 ff 0f 00 ff ff ff ff ff ff 00 0e 00 00 f0 ff f5
 f7 ea 0b 03 05 86 1b 30 f6 ad 27 00 00 00 00 00
 00 00 01 00 00 14 00 00 00 00 00 00 00 00 00 00
 00 00 00 00

00017106  26.20366640   88 da 00 01 a0
00017113  26.20557760   88 d9 00 01 8d
00017120  26.20748480   88 d9 00 01 a2
00017127  26.20939440   88 b3 00 01 00
00017134  26.21130160   88 b3 00 01 00
00017141  26.21320960   80 10 00 02 <28 3f
00017157  26.21706800   88 10 00 02 28 3f
00017164  26.21893440   88 d9 00 01 a2
00017171  40.53844640   80 b3 00 01 <00

00017187
88 b3 00 01 01
88 b3 00 01 01
88 b3 00 01 00
88 b3 00 01 00
88 12 00 02 ff 20
88 14 00 02 f8 28
88 b3 00 01 00 88 b3 00 01 00 88 10 00 02 28 3f
88 d9 00 01 80

00017200
 88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70
 00 00 00 00 28 3f ff 20 f8 28 07 00 ff 00 00 00
 00 20 00 00 3a f2 00 ff ff 00 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00
 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
 00 00 00 00 00 00 e1 14 18 15 00 00 00 00 00 00
 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 ff
 3f 80 68 00 00 00 00 01 0e 00 00 00 00 00 00 00
 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00
 00 00 00 00 00 00 02
 88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00
 ff 0f 00 ff ff ff ff ff ff 00 0e 00 00 f0 ff f5
 f7 ea 0b 03 05 86 1b 30 f6 a2 a0 00 00 00 00 00
 00 00 01 00 00 14 00 00 00 00 00 00 00 00 00 00
 00 00 00 00

00017244  40.55095680   88 da 00 01 a7
00017251  40.55286000   88 d9 00 01 82
00017258  40.55476720   88 d9 00 01 ad
00017265  40.55667760   88 b3 00 01 00
00017272  40.55858480   88 b3 00 01 00
00017279  40.56049200   80 10 00 02 <28 3f  // here comes a worng feed back
00017295  40.56434400   88 10 00 02 28 3f
00017302  40.56622400   88 d9 00 01 ad
00017309  50.15262640   80 b3 00 01 <00
00017325  50.15592640   80 1d 00 01 <22
00017341  50.15975280   80 b3 00 01 <00
00017357  50.16356160   80 da 00 01 <a7
00017373  50.16737760   88 da 00 01 a0 80 10 00 02 <28 3f
00017389  50.17118640   88 da 00 01 a7
00017396  50.17308000   80 da 00 01 <a7
00017412  50.17691760   80 b3 00 01 <00
00017428  50.18073040   80 da 00 01 <a7
00017444  50.18455600   88 da 00 01 a0 80 10 00 02 <28 3f
00017460  50.18836320   88 da 00 01 a7
00017467  50.19025120   80 da 00 01 <a7
00017483  50.19409760   80 b3 00 01 <00

88 b3 00 01 01 //00017499
88 b3 00 01 01
88 b3 00 01 00
88 b3 00 01 00
88 12 00 02 ff 20
88 14 00 02 f8 28
88 b3 00 01 00
88 b3 00 01 00
88 10 00 02 28 3f
88 d9 00 01 80

00017512  50.19981360   88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70
00017514  50.19985200   00 00 00 00 28 3f ff 20 f8 28 07 00 ff 00 00 00
00017516  50.19989040   00 20 00 00 3a f2 00 ff ff 00 00 00 00 00 00 00
00017518  50.19992800   00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00
00017520  50.19996640   00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00
00017522  50.20000400   00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
00017524  50.20004160   00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
00017526  50.20007920   00 00 00 00 00 00 e1 14 18 15 00 00 00 00 00 00
00017528  50.20011680   00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 ff
00017530  50.20015520   3f 80 68 00 00 00 00 01 0e 00 00 00 00 00 00 00
00017532  50.20019280   00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00
00017534  50.20021360   00 00 00 00 00 00 02
00017541  50.20363200   88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00
00017543  50.20367040   ff 0f 00 ff ff ff ff ff ff 00 0e 00 00 f0 ff f5
00017545  50.20370880   f7 ea 0b 03 05 86 1b 30 f6 ad a7 00 00 00 00 00
00017547  50.20374560   00 00 01 00 00 14 00 00 00 00 00 00 00 00 00 00
00017549  50.20376160   00 00 00 00

00017556  50.20654240   88 da 00 01 a7
00017563  50.20838240   88 d9 00 01 8d
00017570  50.21029120   88 d9 00 01 ad
00017577  50.21219920   88 b3 00 01 00
00017584  50.21410800   88 b3 00 01 00
00017591  50.21602320   80 10 00 02 <28 3f
00017607  50.21986560   88 10 00 02 28 3f
00017614  50.22176240   88 d9 00 01 ad
*/


  set_double_reg( 0x10,0x20,0x3f);    //00017621

  memcpy(myregs,myinit33,254);    //00017628  pv75dpi_g23


/***************************************************************************/
  printf("1. scan with 75dpi gray (warm up).......................\n");
  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x\\n",
          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36]);
  printf("\n");

  FD=fopen("test75dpig_warmup.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);
/***************************************************************************/

  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x03);               //00017672
  send_moving_seq();

  while(!data_ready((unsigned int*) &size)) sleep(1);

  while(data_ready((unsigned int*) &size)) {
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };
  fclose(FD);

/* here now move back, but we havn't the sequenz */
/* thats a copy from scan_150dpi_16_color */
/*
  set_double_reg(0x10,0x20,0x3b);
  memcpy(myregs,pv150dpi_23,254);
  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);
  write_reg(0xd3,0x03);
  send_moving_seq();

  printf("start moving back...");
  while((read_reg(MOVE_START_STOP) & 0x08) && !(read_reg(MICROSWITCH) & 0x02)) usleep(1000000); //00857736 /00860072  127.62494960   80 b3 00 01 <00
  printf("done\n");
*/

#endif  // TEST
  init_bevore_scan(R_75_g);


  set_double_reg( 0x10,0x20,0x3f);    //00049270

//  memcpy(myregs,pv75dpi_g24,254);    //00049277
  memcpy(myregs,myinit33,254);    //00049277 pv75dpi_g23

/***************************************************************************/
  printf("1. scan with 75 dpi gray.........................................\n");

  /*memcpy(myregs,pv150dpi_move,254);  */

//  myregs[0x62]=(length+1) & 0xff;
//  myregs[0x63]=((length+1) >> 8) & 0xff;

//  length = myregs[0x62] + 1;
//  length = length + ((myregs[0x63] <<8) & 0xFF00);
/*  x1 = myregs[0x66];
  x1 = x1 + ((myregs[0x67] <<8) & 0xFF00);
  x2 = myregs[0x6c];
  x2 = x2 + ((myregs[0x6d] <<8) & 0xFF00);


  myregs[0x33] = 0x00;
  myregs[0x3a] = 0x0e;
  myregs[0x65] = 0x00;
  myregs[0xe1] = 0x02;*/
  printf("scan_region, length:%d, x1:%d, x2:%d \n",length,x1,x2);
  printf("reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x, reg_3A 0x%x, reg_61/60 0x%x%x, reg_63/62 0x%x%x, reg_65 0x%x, reg_E1 0x%x \n",
          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36],myregs[0x3a],myregs[0x61],myregs[0x60],myregs[0x63],myregs[0x62],myregs[0x65],myregs[0xe1]);

//  myregs[0x60]=(x3) & 0xff;
//  myregs[0x61]=((x3) >> 8) & 0xff;

//  printf("reg_10 0x%x, reg_11 0x%x, reg_e1 0x%x, reg_33 0x%x, reg_36 0x%x, reg_3A 0x%x, reg_60 0x%x, reg_61 0x%x, reg_65 0x%x \n",
//          myregs[0x10],myregs[0x11],myregs[0xe1],myregs[0x33],myregs[0x36],myregs[0x3a],myregs[0x60],myregs[0x61],myregs[0x65]);
/***************************************************************************/

#ifdef REG_EDIT
  memcpy(registers,myregs,254);
  printf("Registers for 75dpi gray scan loaded. You con modify it and start the scan with (t)estscan\n");
#else
  FD=fopen("test75dpi_g_1.pnm","w");
  fprintf(FD,"P5\n%d %d\n255\n",x2-x1,length);

  write_bulk_registers(myregs,0xb3,0);
  write_bulk_registers(myregs+0xb3,0x3f,0xb4);

  write_reg(0xd3,0x0e);               //00049321
  send_moving_seq();
  while(!(read_reg(MOVE_START_STOP) & 0x08) ){ printf(".");usleep(1000000);}; //are we there yet?


  while(!data_ready((unsigned int*) &size)) sleep(1);     //00049370
  while(data_ready((unsigned int*) &size)) {
    read_data(size,data);
    data_count=0;
    while(size--) fprintf(FD,"%c",data[data_count++]);
  };
  fclose(FD);
  free(data);

#endif  //REG_EDIT
  return(0);
};



/****************************************************************************/
int gray_init(void){
// full_scan.dump
// new 19.02.2003
/****************************************************************************/
  unsigned int i,n;
  unsigned char buffer2[0x818+4];

  test_reg(0xb3,0x04,200);
  test_reg(0xb0,0x80,201);
  test_double_reg(0x10,0x60,0x1a,202); /*0x68*/
  wait_for_WarmUp( ); /* 80 84 00 0b < 00 00 00 00 00 00 00 00 00 90 02 */
  test_reg(0xda,0x15,203);
  write_reg(0xda,0xa0);
  test_double_reg(0x10,0x60,0x1a,204); /*0x68*/
  write_reg(0xda,0xa7);
  test_reg(0xda,0xa7,205);
  test_three_regs(0x20,0x00,0x00,0x00,206);
  test_reg(0xb1,0x00,207);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
/*
80 00 00 f4<
f5 41 00 00 00 00 00 00 00 00 00 70 00 00 00 00 68 1b ff 3e 00 00 ff 3e 00 00 10 00 00 22 00 00 00 00 00 ff 00 ff 00 00 00 00 00 00 00 00 00 00 00 00 00 00 10 00 00 00 00 02 00 00 00 00 00 00 80 00 00 00 10 00 00 00 00 00 00 00 00 00 00 00 20 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 02 02 00 00 08 00 00 00 00 00 00 10 00 00 00 00 00 00 00 02 08 00 00 00 00 00 00 00 10 20 40 00 00 00 00 00 00 00 00 00 90 02 00 04 00 00 01 1e 00 00 00 00 00 00 00 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00 00 00 00 00 80 00 06 00 00 00 00 00 00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 01 06 1b 10 7a 00 a7 00 00 00 00 00 00 00 00 00 00 0f 00 00 00 00 00 00 00 00 00 00 00 00 00 00
*/

  write_bulk_registers(gray_init_1,0xb3,0);
  write_bulk_registers(gray_init_1+0xb3,0x40,0xb4);
/*
88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70 00 00 00 00 68 1b 00 20 00 20 00 00 00 00 00 00 00 20 00 00 3a f2 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 80 68 00 00 00 00 02 0e 00 00 00 00 00 00 00 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00 00 00 00 00 00 00 02
88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 86 1b 00 00 00 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
*/
  write_bulk_registers(gray_init_2,0xb3,0);
  write_bulk_registers(gray_init_2+0xb3,0x40,0xb4);
/*
88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70 00 00 00 00 68 1b 08 20 00 20 08 00 00 00 00 00 00 20 00 00 3a f2 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 80 68 00 00 00 00 02 0e 00 00 00 00 00 00 00 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00 00 00 00 00 00 00 02
88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 e0 00 00 00 00 00 86 1b 00 00 00 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
*/
  test_double_reg(0x1a,0x00,0x00,208);
  write_bulk_registers(gray_init_3,0xb3,0); /*same as myinit_1? */
  write_bulk_registers(gray_init_3+0xb3,0x40,0xb4);
/*
88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70 00 00 00 00 68 1b 08 20 00 20 08 00 00 00 00 00 00 20 00 00 3a f2 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 80 68 00 00 00 00 02 0e 00 00 00 00 00 00 00 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00 00 00 00 00 00 00 02
88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 e0 00 00 00 00 86 1b 00 00 00 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
*/
  test_double_reg(0x1a,0x00,0x00,209);
  write_bulk_registers(gray_init_4,0xb3,0); /*same as myinit_2? */
  write_bulk_registers(gray_init_4+0xb3,0x40,0xb4);
/*
88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70 00 00 00 00 68 1b 00 21 00 20 00 01 00 00 00 00 00 20 00 00 3a f2 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 80 68 00 00 00 00 02 0e 00 00 00 00 00 00 00 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00 00 00 00 00 00 00 02
88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 e0 00 00 00 86 1b 00 00 00 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
*/
  test_double_reg(0x1a,0x00,0x00,210);
  write_bulk_registers(gray_init_5,0xb3,0); /*same as myinit_3? */
  write_bulk_registers(gray_init_5+0xb3,0x40,0xb4);
/*
88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70 00 00 00 00 68 1b 08 20 00 20 08 00 00 00 00 00 00 20 00 00 3a f2 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 80 68 00 00 00 00 02 0e 00 00 00 00 00 00 00 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00 00 00 00 00 00 00 02
88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 e0 00 00 00 86 1b 00 00 00 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
*/
  test_double_reg(0x1a,0x00,0x00,211);
  set_double_reg(0x10,0x20,0x28);
  set_double_reg(0x10,0x28,0x28);
  write_reg(0xb3,0x02);
  write_reg(0xb3,0x02);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);
  test_reg(0xb1,0x00,212);
  write_bulk_registers(gray_init_6,0xb3,0); /*same as myinit_4? */
  write_bulk_registers(gray_init_6+0xb3,0x40,0xb4);
/*
88 00 00 b3 e5 41 1f 1f 1f 1f 1f 1f 0a 0a 0a 70 00 00 00 00 28 28 ff 20 f8 28 07 00 ff 00 00 00 00 20 00 00 3a f2 00 ff ff 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 10 00 07 00 00 00 00 00 00 00 00 00 20 00 00 00 8c 76 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 e1 14 18 15 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 80 68 00 00 00 00 02 0e 00 00 00 00 00 00 00 00 00 00 00 00 00 00 cc 27 64 00 00 00 00 00 00 00 00 00 00 00 00 02
88 b4 00 40 00 00 00 00 00 00 00 00 00 00 00 00 ff 0f 00 00 f0 ff ff 0f 00 ff f1 ff 00 f0 ff f5 f7 ea 0b 03 05 86 1b 10 f6 80 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
*/
  for (n=0;n<3;n++){
    switch (n){
      case 0: write_reg(0x93,0x06); break;
      case 1: write_reg(0x93,0x02); break;
      case 2: write_reg(0x93,0x01); break;
      default:;
    }
    set_double_reg( 0x91, 0x81, 0x00 );
    callibration_buffer[0]=0x89;
    callibration_buffer[1]=0x00;
    callibration_buffer[2]=0x08;
    callibration_buffer[3]=0x18;
    callibration_buffer[4]=0;
    for(i=5;i<0x818;i++) {
       callibration_buffer[i]=callibration_buffer[i-1]+1;
       if(callibration_buffer[i]==0x61) callibration_buffer[i]=0;
    };
    write_usb(device,0x02,callibration_buffer,0x818+4,TIMEOUT);
    set_double_reg(0x91,0x81,0x00);
    read_bulk_callibriation_data(buffer2,0x0818);
    };

  write_bulk_registers(gray_init_7,0xb3,0); /*same as init_array(5)?*/
  write_bulk_registers(gray_init_7+0xb3,0x40,0xb4);

  for(i=0;i<5;i++) {
    set_double_reg(0x91, 0x00, (unsigned char) i << 4 );
    write_cal_bytes( i );
    set_double_reg(0x91,0x00,0x00);
    read_bulk_callibriation_data(buffer2,0x20);
  };

  test_reg(0xb3,0x00,213);
  test_reg(0x1d,0x22,214);
  write_reg(0xd9,0x80);
  write_reg(0xd9,0xad);
  test_double_reg(0x14,0xf8,0x28,215);
  set_double_reg(0x14,0x78,0x28);
  test_double_reg(0x10,0xa8,0x3f,216);
  test_reg(0xb1,0x00,217);
  test_reg(0xb3,0x00,218);
  test_reg(0xda,0x27,219);
  write_reg(0xda,0xa0);
  test_double_reg(0x10,0xa8,0x3f,220);
  write_reg(0xda,0xa7);
  test_reg(0xda,0xa7,221);
  write_reg(0xd9,0x8d);
  write_reg(0xd9,0xad);
  test_reg(0xb3,0x00,222);
  test_reg(0xb0,0x80,223);
  test_double_reg(0x10,0xa8,0x3b,224);
  wait_for_WarmUp( ); /* 80 84 00 0b <00 00 00 00 00 00 00 00 00 80 68 */
  test_reg(0xda,0xa7,225);
  write_reg(0xda,0xa0);
  test_double_reg(0x10,0xa8,0x3f,226);
  write_reg(0xda,0xa7);
  test_reg(0xda,0xa7,227);
  test_three_regs(0x20,0x3a,0xf2,0x00,228);
  test_reg(0xb1,0x00,229);
  write_reg(0xb3,0x00);
  write_reg(0xb3,0x00);

  return(0);
};
